@@ -19,17 +19,14 @@ int main() {
     double
       lat1 = 40.640, lon1 = -73.779, // JFK
       lat2 =  1.359, lon2 = 103.989; // SIN
-    double s12, azi1, azi2,
-      a12 = geod.Inverse(lat1, lon1, lat2, lon2, s12, azi1, azi2);
-    const GeographicLib::GeodesicLineExact line(geod, lat1, lon1, azi1);
-    // Alternatively
-    // const GeographicLib::GeodesicLineExact line = geod.Line(lat1,lon1,azi1);
+    const GeographicLib::GeodesicLineExact line =
+      geod.InverseLine(lat1, lon1, lat2, lon2);
     double ds0 = 500e3;             // Nominal distance between points = 500 km
-    int num = int(ceil(s12 / ds0)); // The number of intervals
+    int num = int(ceil(line.Distance() / ds0)); // The number of intervals
     cout << fixed << setprecision(3);
     {
       // Use intervals of equal length
-      double ds = s12 / num;
+      double ds = line.Distance() / num;
       for (int i = 0; i <= num; ++i) {
         double lat, lon;
        line.Position(i * ds, lat, lon);
@@ -38,7 +35,7 @@ int main() {
     }
     {
       // Slightly faster, use intervals of equal arc length
-      double da = a12 / num;
+      double da = line.Arc() / num;
       for (int i = 0; i <= num; ++i) {
         double lat, lon;
        line.ArcPosition(i * da, lat, lon);
@@ -50,5 +47,4 @@ int main() {
     cerr << "Caught exception: " << e.what() << "\n";
     return 1;
   }
-  return 0;
 }
