@@ -4,7 +4,7 @@
  *
  * Copyright (c) Charles Karney (2008-2015) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
- * http://geographiclib.sourceforge.net/
+ * https://geographiclib.sourceforge.io/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_MGRS_HPP)
@@ -344,14 +344,6 @@ namespace GeographicLib {
      **********************************************************************/
     static void Check();
 
-    /// \cond SKIP
-    /**
-     * <b>DEPRECATED</b>
-     * @return \e r the inverse flattening of the WGS84 ellipsoid.
-     **********************************************************************/
-    static Math::real InverseFlattening()
-    { return UTMUPS::InverseFlattening(); }
-    /// \endcond
   };
 
 } // namespace GeographicLib
