@@ -2,9 +2,9 @@
  * \file Utility.hpp
  * \brief Header for GeographicLib::Utility class
  *
- * Copyright (c) Charles Karney (2011-2015) <charles@karney.com> and licensed
+ * Copyright (c) Charles Karney (2011-2017) <charles@karney.com> and licensed
  * under the MIT/X11 License.  For more information, see
- * http://geographiclib.sourceforge.net/
+ * https://geographiclib.sourceforge.io/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_UTILITY_HPP)
@@ -188,27 +188,27 @@ namespace GeographicLib {
       const char* digits = "0123456789";
       std::string::size_type p1 = s.find_first_not_of(digits);
       if (p1 == std::string::npos)
-        y1 = num<int>(s);
+        y1 = val<int>(s);
       else if (s[p1] != '-')
         throw GeographicErr("Delimiter not hyphen in date " + s);
       else if (p1 == 0)
         throw GeographicErr("Empty year field in date " + s);
       else {
-        y1 = num<int>(s.substr(0, p1));
+        y1 = val<int>(s.substr(0, p1));
         if (++p1 == s.size())
           throw GeographicErr("Empty month field in date " + s);
         std::string::size_type p2 = s.find_first_not_of(digits, p1);
         if (p2 == std::string::npos)
-          m1 = num<int>(s.substr(p1));
+          m1 = val<int>(s.substr(p1));
         else if (s[p2] != '-')
           throw GeographicErr("Delimiter not hyphen in date " + s);
         else if (p2 == p1)
           throw GeographicErr("Empty month field in date " + s);
         else {
-          m1 = num<int>(s.substr(p1, p2 - p1));
+          m1 = val<int>(s.substr(p1, p2 - p1));
           if (++p2 == s.size())
             throw GeographicErr("Empty day field in date " + s);
-          d1 = num<int>(s.substr(p2));
+          d1 = val<int>(s.substr(p2));
         }
       }
       y = y1; m = m1; d = d1;
@@ -251,10 +251,9 @@ namespace GeographicLib {
      **********************************************************************/
     template<typename T> static T fractionalyear(const std::string& s) {
       try {
-        return num<T>(s);
-      }
-      catch (const std::exception&) {
+        return val<T>(s);
       }
+      catch (const std::exception&) {}
       int y, m, d;
       date(s, y, m, d);
       int t = day(y, m, d, true);
@@ -316,26 +315,51 @@ namespace GeographicLib {
     }
 
     /**
-     * Convert a string to an object of type T.
+     * Trim the white space from the beginning and end of a string.
      *
-     * @tparam T the type of the return value.
-     * @param[in] s the string to be converted.
-     * @exception GeographicErr is \e s is not readable as a T.
-     * @return object of type T
-     *
-     * White space at the beginning and end of \e s is ignored.
+     * @param[in] s the string to be trimmed
+     * @return the trimmed string
      **********************************************************************/
-    template<typename T> static T num(const std::string& s) {
-      T x;
-      std::string errmsg;
-      std::string::size_type
+    static std::string trim(const std::string& s) {
+      unsigned
         beg = 0,
         end = unsigned(s.size());
       while (beg < end && isspace(s[beg]))
         ++beg;
       while (beg < end && isspace(s[end - 1]))
         --end;
-      std::string t = s.substr(beg, end-beg);
+      return std::string(s, beg, end-beg);
+    }
+
+    /**
+     * Convert a string to type T.
+     *
+     * @tparam T the type of the return value.
+     * @param[in] s the string to be converted.
+     * @exception GeographicErr is \e s is not readable as a T.
+     * @return object of type T.
+     *
+     * White space at the beginning and end of \e s is ignored.
+     *
+     * Special handling is provided for some types.
+     *
+     * If T is a floating point type, then inf and nan are recognized.
+     *
+     * If T is bool, then \e s should either be string a representing 0 (false)
+     * or 1 (true) or one of the strings
+     * - "false", "f", "nil", "no", "n", "off", or "" meaning false,
+     * - "true", "t", "yes", "y", or "on" meaning true;
+     * .
+     * case is ignored.
+     *
+     * If T is std::string, then \e s is returned (with the white space at the
+     * beginning and end removed).
+     **********************************************************************/
+    template<typename T> static T val(const std::string& s) {
+      // If T is bool, then the specialization val<bool>() defined below is
+      // used.
+      T x;
+      std::string errmsg, t(trim(s));
       do {                     // Executed once (provides the ability to break)
         std::istringstream is(t);
         if (!(is >> x)) {
@@ -354,11 +378,20 @@ namespace GeographicLib {
         throw GeographicErr(errmsg);
       return x;
     }
+    /**
+     * \deprecated An old name for val<T>(s).
+     **********************************************************************/
+    template<typename T>
+      // GEOGRAPHICLIB_DEPRECATED("Use new Utility::val<T>(s)")
+      static T num(const std::string& s) {
+      return val<T>(s);
+    }
 
     /**
      * Match "nan" and "inf" (and variants thereof) in a string.
      *
-     * @tparam T the type of the return value.
+     * @tparam T the type of the return value (this should be a floating point
+     *   type).
      * @param[in] s the string to be matched.
      * @return appropriate special value (&plusmn;&infin;, nan) or 0 if none is
      *   found.
@@ -368,9 +401,9 @@ namespace GeographicLib {
     template<typename T> static T nummatch(const std::string& s) {
       if (s.length() < 3)
         return 0;
-      std::string t;
-      t.resize(s.length());
-      std::transform(s.begin(), s.end(), t.begin(), (int(*)(int))std::toupper);
+      std::string t(s);
+      for (std::string::iterator p = t.begin(); p != t.end(); ++p)
+        *p = char(std::toupper(*p));
       for (size_t i = s.length(); i--;)
         t[i] = char(std::toupper(s[i]));
       int sign = t[0] == '-' ? -1 : 1;
@@ -396,7 +429,7 @@ namespace GeographicLib {
      * @exception GeographicErr is \e s is not readable as a fraction of type T.
      * @return object of type T
      *
-     * <b>NOTE</b>: The msys shell under Windows converts arguments which look
+     * \note The msys shell under Windows converts arguments which look
      * like pathnames into their Windows equivalents.  As a result the argument
      * "-1/300" gets mangled into something unrecognizable.  A workaround is to
      * use a floating point number in the numerator, i.e., "-1.0/300".
@@ -405,9 +438,9 @@ namespace GeographicLib {
       std::string::size_type delim = s.find('/');
       return
         !(delim != std::string::npos && delim >= 1 && delim + 2 <= s.size()) ?
-        num<T>(s) :
+        val<T>(s) :
         // delim in [1, size() - 2]
-        num<T>(s.substr(0, delim)) / num<T>(s.substr(delim + 1));
+        val<T>(s.substr(0, delim)) / val<T>(s.substr(delim + 1));
     }
 
     /**
@@ -591,7 +624,7 @@ namespace GeographicLib {
      *   256 (i.e., about 77 decimal digits).
      * @return the resulting number of bits of precision.
      *
-     * This only has an effect when GEOGRAPHICLIB_PRECISION == 5.  The
+     * This only has an effect when GEOGRAPHICLIB_PRECISION = 5.  The
      * precision should only be set once and before calls to any other
      * GeographicLib functions.  (Several functions, for example Math::pi(),
      * cache the return value in a static local variable.  The precision needs
@@ -603,6 +636,50 @@ namespace GeographicLib {
 
   };
 
+  /**
+   * The specialization of Utility::val<T>() for strings.
+   **********************************************************************/
+  template<> inline std::string Utility::val<std::string>(const std::string& s)
+  { return trim(s); }
+
+  /**
+   * The specialization of Utility::val<T>() for bools.
+   **********************************************************************/
+  template<> inline bool Utility::val<bool>(const std::string& s) {
+    std::string t(trim(s));
+    if (t.empty()) return false;
+    bool x;
+    std::istringstream is(t);
+    if (is >> x) {
+      int pos = int(is.tellg()); // Returns -1 at end of string?
+      if (!(pos < 0 || pos == int(t.size())))
+        throw GeographicErr("Extra text " + t.substr(pos) +
+                            " at end of " + t);
+      return x;
+    }
+    for (std::string::iterator p = t.begin(); p != t.end(); ++p)
+      *p = char(std::tolower(*p));
+    switch (t[0]) {             // already checked that t isn't empty
+    case 'f':
+      if (t == "f" || t == "false") return false;
+      break;
+    case 'n':
+      if (t == "n" || t == "nil" || t == "no") return false;
+      break;
+    case 'o':
+      if (t == "off") return false;
+      else if (t == "on") return true;
+      break;
+    case 't':
+      if (t == "t" || t == "true") return true;
+      break;
+    case 'y':
+      if (t == "y" || t == "yes") return true;
+      break;
+    }
+    throw GeographicErr("Cannot decode " + t + " as a bool");
+  }
+
 } // namespace GeographicLib
 
 #if defined(_MSC_VER)
