@@ -4,9 +4,9 @@
  *
  * Run these tests by configuring with cmake and running "make test".
  *
- * Copyright (c) Charles Karney (2015) <charles@karney.com> and licensed under
- * the MIT/X11 License.  For more information, see
- * http://geographiclib.sourceforge.net/
+ * Copyright (c) Charles Karney (2015-2017) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
+ * https://geographiclib.sourceforge.io/
  **********************************************************************/
 
 /** @cond SKIP */
@@ -15,9 +15,14 @@
 #include <stdio.h>
 #include <math.h>
 
+#if defined(_MSC_VER)
+// Squelch warnings about assignment within conditional expression
+#  pragma warning (disable: 4706)
+#endif
+
 double wgs84_a = 6378137, wgs84_f = 1/298.257223563; /* WGS84 */
 
-int assertEquals(double x, double y, double d) {
+static int assertEquals(double x, double y, double d) {
   if (fabs(x - y) <= d)
     return 0;
   printf("assertEquals fails: %.7g != %.7g +/- %.7g\n", x, y, d);
@@ -107,7 +112,7 @@ double testcases[20][12] = {
    13487015.8381145492, 121.294026715742277, 5481428.9945736388,
    -0.51527225545373252, -0.51556587964721788, 104679964020340.318}};
 
-int testinverse() {
+static int testinverse() {
   double lat1, lon1, azi1, lat2, lon2, azi2, s12, a12, m12, M12, M21, S12;
   double azi1a, azi2a, s12a, a12a, m12a, M12a, M21a, S12a;
   struct geod_geodesic g;
@@ -132,7 +137,7 @@ int testinverse() {
   return result;
 }
 
-int testdirect() {
+static int testdirect() {
   double lat1, lon1, azi1, lat2, lon2, azi2, s12, a12, m12, M12, M21, S12;
   double lat2a, lon2a, azi2a, a12a, m12a, M12a, M21a, S12a;
   struct geod_geodesic g;
@@ -159,7 +164,7 @@ int testdirect() {
   return result;
 }
 
-int testarcdirect() {
+static int testarcdirect() {
   double lat1, lon1, azi1, lat2, lon2, azi2, s12, a12, m12, M12, M21, S12;
   double lat2a, lon2a, azi2a, s12a, m12a, M12a, M21a, S12a;
   struct geod_geodesic g;
@@ -185,7 +190,7 @@ int testarcdirect() {
   return result;
 }
 
-int GeodSolve0() {
+static int GeodSolve0() {
   double azi1, azi2, s12;
   struct geod_geodesic g;
   int result = 0;
@@ -197,7 +202,7 @@ int GeodSolve0() {
   return result;
 }
 
-int GeodSolve1() {
+static int GeodSolve1() {
   double lat2, lon2, azi2;
   struct geod_geodesic g;
   int result = 0;
@@ -210,7 +215,7 @@ int GeodSolve1() {
   return result;
 }
 
-int GeodSolve2() {
+static int GeodSolve2() {
   /* Check fix for antipodal prolate bug found 2010-09-04 */
   double azi1, azi2, s12;
   struct geod_geodesic g;
@@ -227,7 +232,7 @@ int GeodSolve2() {
   return result;
 }
 
-int GeodSolve4() {
+static int GeodSolve4() {
   /* Check fix for short line bug found 2010-05-21 */
   double s12;
   struct geod_geodesic g;
@@ -239,7 +244,7 @@ int GeodSolve4() {
   return result;
 }
 
-int GeodSolve5() {
+static int GeodSolve5() {
   /* Check fix for point2=pole bug found 2010-05-03 */
   double lat2, lon2, azi2;
   struct geod_geodesic g;
@@ -249,7 +254,7 @@ int GeodSolve5() {
   result += assertEquals(lat2, 90, 0.5e-5);
   if (lon2 < 0) {
     result += assertEquals(lon2, -150, 0.5e-5);
-    result += assertEquals(azi2, -180, 0.5e-5);
+    result += assertEquals(fabs(azi2), 180, 0.5e-5);
   } else {
     result += assertEquals(lon2, 30, 0.5e-5);
     result += assertEquals(azi2, 0, 0.5e-5);
@@ -257,7 +262,7 @@ int GeodSolve5() {
   return result;
 }
 
-int GeodSolve6() {
+static int GeodSolve6() {
   /* Check fix for volatile sbet12a bug found 2011-06-25 (gcc 4.4.4
    * x86 -O3).  Found again on 2012-03-27 with tdm-mingw32 (g++ 4.6.1). */
   double s12;
@@ -276,7 +281,7 @@ int GeodSolve6() {
   return result;
 }
 
-int GeodSolve9() {
+static int GeodSolve9() {
   /* Check fix for volatile x bug found 2011-06-25 (gcc 4.4.4 x86 -O3) */
   double s12;
   struct geod_geodesic g;
@@ -288,7 +293,7 @@ int GeodSolve9() {
   return result;
 }
 
-int GeodSolve10() {
+static int GeodSolve10() {
   /* Check fix for adjust tol1_ bug found 2011-06-25 (Visual Studio
    * 10 rel + debug) */
   double s12;
@@ -301,7 +306,7 @@ int GeodSolve10() {
   return result;
 }
 
-int GeodSolve11() {
+static int GeodSolve11() {
   /* Check fix for bet2 = -bet1 bug found 2011-06-25 (Visual Studio
    * 10 rel + debug) */
   double s12;
@@ -314,7 +319,7 @@ int GeodSolve11() {
   return result;
 }
 
-int GeodSolve12() {
+static int GeodSolve12() {
   /* Check fix for inverse geodesics on extreme prolate/oblate
    * ellipsoids Reported 2012-08-29 Stefan Guenther
    * <stefan.gunther@embl.de>; fixed 2012-10-07 */
@@ -329,11 +334,15 @@ int GeodSolve12() {
   return result;
 }
 
-int GeodSolve14() {
+static int GeodSolve14() {
   /* Check fix for inverse ignoring lon12 = nan */
-  double azi1, azi2, s12, nan = sqrt(-1.0);
+  double azi1, azi2, s12, nan;
   struct geod_geodesic g;
   int result = 0;
+  {
+    double minus1 = -1;
+    nan = sqrt(minus1);
+  }
   geod_init(&g, wgs84_a, wgs84_f);
   geod_inverse(&g, 0, 0, 1, nan, &s12, &azi1, &azi2);
   result += azi1 == azi1 ? 1 : 0;
@@ -342,7 +351,7 @@ int GeodSolve14() {
   return result;
 }
 
-int GeodSolve15() {
+static int GeodSolve15() {
   /* Initial implementation of Math::eatanhe was wrong for e^2 < 0.  This
    * checks that this is fixed. */
   double S12;
@@ -355,7 +364,7 @@ int GeodSolve15() {
   return result;
 }
 
-int GeodSolve17() {
+static int GeodSolve17() {
   /* Check fix for LONG_UNROLL bug found on 2015-05-07 */
   double lat2, lon2, azi2;
   struct geod_geodesic g;
@@ -384,7 +393,7 @@ int GeodSolve17() {
   return result;
 }
 
-int GeodSolve26() {
+static int GeodSolve26() {
   /* Check 0/0 problem with area calculation on sphere 2015-09-08 */
   double S12;
   struct geod_geodesic g;
@@ -395,7 +404,7 @@ int GeodSolve26() {
   return result;
 }
 
-int GeodSolve28() {
+static int GeodSolve28() {
   /* Check for bad placement of assignment of r.a12 with |f| > 0.01 (bug in
    * Java implementation fixed on 2015-05-19). */
   double a12;
@@ -407,7 +416,7 @@ int GeodSolve28() {
   return result;
 }
 
-int GeodSolve33() {
+static int GeodSolve33() {
   /* Check max(-0.0,+0.0) issues 2015-08-22 (triggered by bugs in Octave --
    * sind(-0.0) = +0.0 -- and in some version of Visual Studio --
    * fmod(-0.0, 360.0) = +0.0. */
@@ -425,11 +434,11 @@ int GeodSolve33() {
   result += assertEquals(s12, 19980862, 0.5);
   geod_inverse(&g, 0, 0, 0, 180, &s12, &azi1, &azi2);
   result += assertEquals(azi1, 0.00000, 0.5e-5);
-  result += assertEquals(azi2, -180.00000, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180.00000, 0.5e-5);
   result += assertEquals(s12, 20003931, 0.5);
   geod_inverse(&g, 0, 0, 1, 180, &s12, &azi1, &azi2);
   result += assertEquals(azi1, 0.00000, 0.5e-5);
-  result += assertEquals(azi2, -180.00000, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180.00000, 0.5e-5);
   result += assertEquals(s12, 19893357, 0.5);
   geod_init(&g, 6.4e6, 0);
   geod_inverse(&g, 0, 0, 0, 179, &s12, &azi1, &azi2);
@@ -438,11 +447,11 @@ int GeodSolve33() {
   result += assertEquals(s12, 19994492, 0.5);
   geod_inverse(&g, 0, 0, 0, 180, &s12, &azi1, &azi2);
   result += assertEquals(azi1, 0.00000, 0.5e-5);
-  result += assertEquals(azi2, -180.00000, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180.00000, 0.5e-5);
   result += assertEquals(s12, 20106193, 0.5);
   geod_inverse(&g, 0, 0, 1, 180, &s12, &azi1, &azi2);
   result += assertEquals(azi1, 0.00000, 0.5e-5);
-  result += assertEquals(azi2, -180.00000, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180.00000, 0.5e-5);
   result += assertEquals(s12, 19994492, 0.5);
   geod_init(&g, 6.4e6, -1/300.0);
   geod_inverse(&g, 0, 0, 0, 179, &s12, &azi1, &azi2);
@@ -459,18 +468,22 @@ int GeodSolve33() {
   result += assertEquals(s12, 20082617, 0.5);
   geod_inverse(&g, 0, 0, 1, 180, &s12, &azi1, &azi2);
   result += assertEquals(azi1, 0.00000, 0.5e-5);
-  result += assertEquals(azi2, -180.00000, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180.00000, 0.5e-5);
   result += assertEquals(s12, 20027270, 0.5);
 
   return result;
 }
 
-int GeodSolve55() {
+static int GeodSolve55() {
   /* Check fix for nan + point on equator or pole not returning all nans in
    * Geodesic::Inverse, found 2015-09-23. */
-  double azi1, azi2, s12, nan = sqrt(-1.0);
+  double azi1, azi2, s12, nan;
   struct geod_geodesic g;
   int result = 0;
+  {
+    double minus1 = -1;
+    nan = sqrt(minus1);
+  }
   geod_init(&g, wgs84_a, wgs84_f);
   geod_inverse(&g, nan, 0, 0, 90, &s12, &azi1, &azi2);
   result += azi1 == azi1 ? 1 : 0;
@@ -483,7 +496,129 @@ int GeodSolve55() {
   return result;
 }
 
-void planimeter(const struct geod_geodesic* g, double points[][2], int N,
+static int GeodSolve59() {
+  /* Check for points close with longitudes close to 180 deg apart. */
+  double azi1, azi2, s12;
+  struct geod_geodesic g;
+  int result = 0;
+  geod_init(&g, wgs84_a, wgs84_f);
+  geod_inverse(&g, 5, 0.00000000000001, 10, 180, &s12, &azi1, &azi2);
+  result += assertEquals(azi1, 0.000000000000035, 1.5e-14);
+  result += assertEquals(azi2, 179.99999999999996, 1.5e-14);
+  result += assertEquals(s12, 18345191.174332713, 2.5e-9);
+  return result;
+}
+
+static int GeodSolve61() {
+  /* Make sure small negative azimuths are west-going */
+  double lat2, lon2, azi2;
+  struct geod_geodesic g;
+  struct geod_geodesicline l;
+  int result = 0;
+  unsigned flags = GEOD_LONG_UNROLL;
+  geod_init(&g, wgs84_a, wgs84_f);
+  geod_gendirect(&g, 45, 0, -0.000000000000000003, flags, 1e7,
+                 &lat2, &lon2, &azi2, 0, 0, 0, 0, 0);
+  result += assertEquals(lat2, 45.30632, 0.5e-5);
+  result += assertEquals(lon2, -180, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180, 0.5e-5);
+  geod_inverseline(&l, &g, 45, 0, 80, -0.000000000000000003, 0);
+  geod_genposition(&l, flags, 1e7, &lat2, &lon2, &azi2, 0, 0, 0, 0, 0);
+  result += assertEquals(lat2, 45.30632, 0.5e-5);
+  result += assertEquals(lon2, -180, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180, 0.5e-5);
+  return result;
+}
+
+static int GeodSolve65() {
+  /* Check for bug in east-going check in GeodesicLine (needed to check for
+   * sign of 0) and sign error in area calculation due to a bogus override of
+   * the code for alp12.  Found/fixed on 2015-12-19. */
+  double lat2, lon2, azi2, s12, a12, m12, M12, M21, S12;
+  struct geod_geodesic g;
+  struct geod_geodesicline l;
+  int result = 0;
+  unsigned flags = GEOD_LONG_UNROLL, caps = GEOD_ALL;
+  geod_init(&g, wgs84_a, wgs84_f);
+  geod_inverseline(&l, &g, 30, -0.000000000000000001, -31, 180, caps);
+  a12 = geod_genposition(&l, flags, 1e7,
+                         &lat2, &lon2, &azi2, &s12, &m12, &M12, &M21, &S12);
+  result += assertEquals(lat2, -60.23169, 0.5e-5);
+  result += assertEquals(lon2, -0.00000, 0.5e-5);
+  result += assertEquals(fabs(azi2), 180.00000, 0.5e-5);
+  result += assertEquals(s12, 10000000, 0.5);
+  result += assertEquals(a12, 90.06544, 0.5e-5);
+  result += assertEquals(m12, 6363636, 0.5);
+  result += assertEquals(M12, -0.0012834, 0.5e-7);
+  result += assertEquals(M21, 0.0013749, 0.5e-7);
+  result += assertEquals(S12, 0, 0.5);
+  a12 = geod_genposition(&l, flags, 2e7,
+                         &lat2, &lon2, &azi2, &s12, &m12, &M12, &M21, &S12);
+  result += assertEquals(lat2, -30.03547, 0.5e-5);
+  result += assertEquals(lon2, -180.00000, 0.5e-5);
+  result += assertEquals(azi2, -0.00000, 0.5e-5);
+  result += assertEquals(s12, 20000000, 0.5);
+  result += assertEquals(a12, 179.96459, 0.5e-5);
+  result += assertEquals(m12, 54342, 0.5);
+  result += assertEquals(M12, -1.0045592, 0.5e-7);
+  result += assertEquals(M21, -0.9954339, 0.5e-7);
+  result += assertEquals(S12, 127516405431022.0, 0.5);
+  return result;
+}
+
+static int GeodSolve67() {
+  /* Check for InverseLine if line is slightly west of S and that s13 is
+     correctly set. */
+  double lat2, lon2, azi2;
+  struct geod_geodesic g;
+  struct geod_geodesicline l;
+  int result = 0;
+  unsigned flags = GEOD_LONG_UNROLL;
+  geod_init(&g, wgs84_a, wgs84_f);
+  geod_inverseline(&l, &g, -5, -0.000000000000002, -10, 180, 0);
+  geod_genposition(&l, flags, 2e7, &lat2, &lon2, &azi2, 0, 0, 0, 0, 0);
+  result += assertEquals(lat2, 4.96445, 0.5e-5);
+  result += assertEquals(lon2, -180.00000, 0.5e-5);
+  result += assertEquals(azi2, -0.00000, 0.5e-5);
+  geod_genposition(&l, flags, 0.5 * l.s13, &lat2, &lon2, &azi2, 0, 0, 0, 0, 0);
+  result += assertEquals(lat2, -87.52461, 0.5e-5);
+  result += assertEquals(lon2, -0.00000, 0.5e-5);
+  result += assertEquals(azi2, -180.00000, 0.5e-5);
+  return result;
+}
+
+static int GeodSolve71() {
+  /* Check that DirectLine sets s13. */
+  double lat2, lon2, azi2;
+  struct geod_geodesic g;
+  struct geod_geodesicline l;
+  int result = 0;
+  geod_init(&g, wgs84_a, wgs84_f);
+  geod_directline(&l, &g, 1, 2, 45, 1e7, 0);
+  geod_position(&l, 0.5 * l.s13, &lat2, &lon2, &azi2);
+  result += assertEquals(lat2, 30.92625, 0.5e-5);
+  result += assertEquals(lon2, 37.54640, 0.5e-5);
+  result += assertEquals(azi2, 55.43104, 0.5e-5);
+  return result;
+}
+
+static int GeodSolve73() {
+  /* Check for backwards from the pole bug reported by Anon on 2016-02-13.
+   * This only affected the Java implementation.  It was introduced in Java
+   * version 1.44 and fixed in 1.46-SNAPSHOT on 2016-01-17. */
+  double lat2, lon2, azi2;
+  struct geod_geodesic g;
+  int result = 0;
+  geod_init(&g, wgs84_a, wgs84_f);
+  geod_direct(&g, 90, 10, 180, -1e6,
+              &lat2, &lon2, &azi2);
+  result += assertEquals(lat2, 81.04623, 0.5e-5);
+  result += assertEquals(lon2, -170, 0.5e-5);
+  result += assertEquals(azi2, 0, 0.5e-5);
+  return result;
+}
+
+static void planimeter(const struct geod_geodesic* g, double points[][2], int N,
                 double* perimeter, double* area) {
   struct geod_polygon p;
   int i;
@@ -493,7 +628,7 @@ void planimeter(const struct geod_geodesic* g, double points[][2], int N,
   geod_polygon_compute(g, &p, 0, 1, area, perimeter);
 }
 
-void polylength(const struct geod_geodesic* g, double points[][2], int N,
+static void polylength(const struct geod_geodesic* g, double points[][2], int N,
                 double* perimeter) {
   struct geod_polygon p;
   int i;
@@ -503,7 +638,27 @@ void polylength(const struct geod_geodesic* g, double points[][2], int N,
   geod_polygon_compute(g, &p, 0, 1, 0, perimeter);
 }
 
-int Planimeter0() {
+static int GeodSolve74() {
+  /* Check fix for inaccurate areas, bug introduced in v1.46, fixed
+     2015-10-16. */
+  double a12, s12, azi1, azi2, m12, M12, M21, S12;
+  struct geod_geodesic g;
+  int result = 0;
+  geod_init(&g, wgs84_a, wgs84_f);
+  a12 = geod_geninverse(&g, 54.1589, 15.3872, 54.1591, 15.3877,
+                        &s12, &azi1, &azi2, &m12, &M12, &M21, &S12);
+  result += assertEquals(azi1, 55.723110355, 5e-9);
+  result += assertEquals(azi2, 55.723515675, 5e-9);
+  result += assertEquals(s12,  39.527686385, 5e-9);
+  result += assertEquals(a12,   0.000355495, 5e-9);
+  result += assertEquals(m12,  39.527686385, 5e-9);
+  result += assertEquals(M12,   0.999999995, 5e-9);
+  result += assertEquals(M21,   0.999999995, 5e-9);
+  result += assertEquals(S12, 286698586.30197, 5e-4);
+  return result;
+}
+
+static int Planimeter0() {
   /* Check fix for pole-encircling bug found 2011-03-16 */
   double pa[4][2] = {{89, 0}, {89, 90}, {89, 180}, {89, 270}};
   double pb[4][2] = {{-89, 0}, {-89, 90}, {-89, 180}, {-89, 270}};
@@ -536,7 +691,7 @@ int Planimeter0() {
   return result;
 }
 
-int Planimeter5() {
+static int Planimeter5() {
   /* Check fix for Planimeter pole crossing bug found 2011-06-24 */
   double points[3][2] = {{89, 0.1}, {89, 90.1}, {89, -179.9}};
   struct geod_geodesic g;
@@ -549,7 +704,7 @@ int Planimeter5() {
   return result;
 }
 
-int Planimeter6() {
+static int Planimeter6() {
   /* Check fix for Planimeter lon12 rounding bug found 2012-12-03 */
   double pa[3][2] = {{9, -0.00000000000001}, {9, 180}, {9, 0}};
   double pb[3][2] = {{9, 0.00000000000001}, {9, 0}, {9, 180}};
@@ -575,7 +730,7 @@ int Planimeter6() {
   return result;
 }
 
-int Planimeter12() {
+static int Planimeter12() {
   /* Area of arctic circle (not really -- adjunct to rhumb-area test) */
   double points[2][2] = {{66.562222222, 0}, {66.562222222, 180}};
   struct geod_geodesic g;
@@ -588,7 +743,7 @@ int Planimeter12() {
   return result;
 }
 
-int Planimeter13() {
+static int Planimeter13() {
   /* Check encircling pole twice */
   double points[6][2] = {{89,-360}, {89,-240}, {89,-120},
                          {89,0}, {89,120}, {89,240}};
@@ -624,6 +779,13 @@ int main() {
   if ((i = GeodSolve28())) {++n; printf("GeodSolve28 fail: %d\n", i);}
   if ((i = GeodSolve33())) {++n; printf("GeodSolve33 fail: %d\n", i);}
   if ((i = GeodSolve55())) {++n; printf("GeodSolve55 fail: %d\n", i);}
+  if ((i = GeodSolve59())) {++n; printf("GeodSolve59 fail: %d\n", i);}
+  if ((i = GeodSolve61())) {++n; printf("GeodSolve61 fail: %d\n", i);}
+  if ((i = GeodSolve65())) {++n; printf("GeodSolve65 fail: %d\n", i);}
+  if ((i = GeodSolve67())) {++n; printf("GeodSolve67 fail: %d\n", i);}
+  if ((i = GeodSolve71())) {++n; printf("GeodSolve71 fail: %d\n", i);}
+  if ((i = GeodSolve73())) {++n; printf("GeodSolve73 fail: %d\n", i);}
+  if ((i = GeodSolve74())) {++n; printf("GeodSolve74 fail: %d\n", i);}
   if ((i = Planimeter0())) {++n; printf("Planimeter0 fail: %d\n", i);}
   if ((i = Planimeter5())) {++n; printf("Planimeter5 fail: %d\n", i);}
   if ((i = Planimeter6())) {++n; printf("Planimeter6 fail: %d\n", i);}
