@@ -27,11 +27,13 @@ function geographiclib_test
   i = GeodSolve59; if i, n=n+1; fprintf('GeodSolve59 fail: %d\n', i); end
   i = GeodSolve61; if i, n=n+1; fprintf('GeodSolve61 fail: %d\n', i); end
   i = GeodSolve73; if i, n=n+1; fprintf('GeodSolve73 fail: %d\n', i); end
+  i = GeodSolve74; if i, n=n+1; fprintf('GeodSolve74 fail: %d\n', i); end
   i = Planimeter0; if i, n=n+1; fprintf('Planimeter0 fail: %d\n', i); end
   i = Planimeter5; if i, n=n+1; fprintf('Planimeter5 fail: %d\n', i); end
   i = Planimeter6; if i, n=n+1; fprintf('Planimeter6 fail: %d\n', i); end
   i = Planimeter12; if i, n=n+1; fprintf('Planimeter12 fail: %d\n', i); end
   i = Planimeter13; if i, n=n+1; fprintf('Planimeter13 fail: %d\n', i); end
+  i = geodreckon0; if i, n=n+1; fprintf('geodreckon0 fail: %d\n', i); end
   i = gedistance0; if i, n=n+1; fprintf('gedistance0 fail: %d\n', i); end
   i = tranmerc0; if i, n=n+1; fprintf('tranmerc0 fail: %d\n', i); end
   i = mgrs0; if i, n=n+1; fprintf('mgrs0 fail: %d\n', i); end
@@ -225,7 +227,7 @@ function n = GeodSolve5
   n = n + assertEquals(lat2, 90, 0.5e-5);
   if lon2 < 0
     n = n + assertEquals(lon2, -150, 0.5e-5);
-    n = n + assertEquals(azi2, -180, 0.5e-5);
+    n = n + assertEquals(abs(azi2), 180, 0.5e-5);
   else
     n = n + assertEquals(lon2, 30, 0.5e-5);
     n = n + assertEquals(azi2, 0, 0.5e-5);
@@ -348,11 +350,11 @@ function n = GeodSolve33
   n = n + assertEquals(s12, 19980862, 0.5);
   [s12, azi1, azi2] = geoddistance(0, 0, 0, 180);
   n = n + assertEquals(azi1, 0.00000, 0.5e-5);
-  n = n + assertEquals(azi2, -180.00000, 0.5e-5);
+  n = n + assertEquals(abs(azi2), 180.00000, 0.5e-5);
   n = n + assertEquals(s12, 20003931, 0.5);
   [s12, azi1, azi2] = geoddistance(0, 0, 1, 180);
   n = n + assertEquals(azi1, 0.00000, 0.5e-5);
-  n = n + assertEquals(azi2, -180.00000, 0.5e-5);
+  n = n + assertEquals(abs(azi2), 180.00000, 0.5e-5);
   n = n + assertEquals(s12, 19893357, 0.5);
   ell = ellipsoid(6.4e6, 0);
   [s12, azi1, azi2] = geoddistance(0, 0, 0, 179, ell);
@@ -361,11 +363,11 @@ function n = GeodSolve33
   n = n + assertEquals(s12, 19994492, 0.5);
   [s12, azi1, azi2] = geoddistance(0, 0, 0, 180, ell);
   n = n + assertEquals(azi1, 0.00000, 0.5e-5);
-  n = n + assertEquals(azi2, -180.00000, 0.5e-5);
+  n = n + assertEquals(abs(azi2), 180.00000, 0.5e-5);
   n = n + assertEquals(s12, 20106193, 0.5);
   [s12, azi1, azi2] = geoddistance(0, 0, 1, 180, ell);
   n = n + assertEquals(azi1, 0.00000, 0.5e-5);
-  n = n + assertEquals(azi2, -180.00000, 0.5e-5);
+  n = n + assertEquals(abs(azi2), 180.00000, 0.5e-5);
   n = n + assertEquals(s12, 19994492, 0.5);
   ell = ellipsoid(6.4e6, -1/300.0);
   [s12, azi1, azi2] = geoddistance(0, 0, 0, 179, ell);
@@ -382,7 +384,7 @@ function n = GeodSolve33
   n = n + assertEquals(s12, 20082617, 0.5);
   [s12, azi1, azi2] = geoddistance(0, 0, 1, 180, ell);
   n = n + assertEquals(azi1, 0.00000, 0.5e-5);
-  n = n + assertEquals(azi2, -180.00000, 0.5e-5);
+  n = n + assertEquals(abs(azi2), 180.00000, 0.5e-5);
   n = n + assertEquals(s12, 20027270, 0.5);
   % Check also octave-specific versions of this problem.
   % In 1.44 this returned [-2.0004e+07, -2.0004e+07, 0.0000e+00, 0.0000e+00]
@@ -419,7 +421,7 @@ function n = GeodSolve61
   [lat2, lon2, azi2] = geodreckon(45, 0, 1e7, -0.000000000000000003, 2);
   n = n + assertEquals(lat2, 45.30632, 0.5e-5);
   n = n + assertEquals(lon2, -180, 0.5e-5);
-  n = n + assertEquals(azi2, -180, 0.5e-5);
+  n = n + assertEquals(abs(azi2), 180, 0.5e-5);
 end
 
 function n = GeodSolve73
@@ -433,6 +435,22 @@ function n = GeodSolve73
   n = n + assertEquals(azi2, 0, 0.5e-5);
 end
 
+function n = GeodSolve74
+% Check fix for inaccurate areas, bug introduced in v1.46, fixed
+% 2015-10-16.
+  n = 0;
+  [s12, azi1, azi2, S12, m12, M12, M21, a12] = ...
+      geoddistance(54.1589, 15.3872, 54.1591, 15.3877);
+  n = n + assertEquals(azi1, 55.723110355, 5e-9);
+  n = n + assertEquals(azi2, 55.723515675, 5e-9);
+  n = n + assertEquals(s12,  39.527686385, 5e-9);
+  n = n + assertEquals(a12,   0.000355495, 5e-9);
+  n = n + assertEquals(m12,  39.527686385, 5e-9);
+  n = n + assertEquals(M12,   0.999999995, 5e-9);
+  n = n + assertEquals(M21,   0.999999995, 5e-9);
+  n = n + assertEquals(S12, 286698586.30197, 5e-4);
+end
+
 function n = Planimeter0
 % Check fix for pole-encircling bug found 2011-03-16
   n = 0;
@@ -508,6 +526,20 @@ function n = Planimeter13
   n = n + assertEquals(area, 32415230256.0, 1);
 end
 
+function n = geodreckon0
+% Check mixed array size bugs
+  n = 0;
+  % lat1 is an array, azi1 is a scalar: 2015-08-10
+  [~, ~, ~, S12] = geodreckon([10 20], 0, 0, 0);
+  if length(S12) ~= 2, n = n+1; end
+  % scalar args except s12 is empty: 2017-03-26
+  [~, ~, ~, S12] = geodreckon(10, 0, [], 0);
+  if ~isempty(S12), n = n+1; end
+  % atan2dx needs to accommodate scalar + array arguments: 2017-03-27
+  lat2 = geodreckon(3, 4, [1, 2], 90);
+  if length(lat2) ~= 2, n = n+1; end
+end
+
 function n = gedistance0
 % gedistance(0, 0, 0, 100) wrongly return nan; 2015-09-23
   n = 0;
